package util

import (
	"fmt"
	"github.com/antonholmquist/jason"
	"io/ioutil"
	//	"log"
	//	"os"
	//	"bytes"
	"strings"
)

type Config struct {
	// The JSON object of what was read
	LoadedJson     jason.Object
	Options        jason.Object
	FeatureTPHome  bool
	FeatureTPVisit bool
	Users          []*User
	U              User
}

var c *Config

func LoadConfig(mm *MessageManager) {
	c = new(Config)
	config_str, err := ioutil.ReadFile("mc_man.config")
	if err == nil {
		j, _ := jason.NewObjectFromBytes(config_str)
		o, _ := j.GetObjectArray("options")

		// Add the "Stop" listener
		mm.listeners = append(mm.listeners, func(i *Message) bool {
			if i.User.IsOp && i.Text == "!stop\n" {
				mm.Output("stop")
				return true
			}
			return false
		})

		for _, option := range o {
			opt_name, _ := option.GetString("name")
			opt_enabled, _ := option.GetBoolean("enabled")
			if opt_name == "home" {
				c.FeatureTPHome = opt_enabled
				// Add the listener for "!set home"
				// Add !set home listener
				mm.listeners = append(mm.listeners, func(i *Message) bool {
					if i.User.Name != "" && i.Text == "!set home\n" {
						mm.tempListeners = append(mm.tempListeners, func(inp *Message) bool {
							listen_for := "Teleported " + i.User.Name + " to "
							if inp.User.Name == "" && strings.Contains(inp.Text, listen_for) {
								// Found the text
								r := strings.Split(inp.Text, listen_for)
								if len(r) > 0 {
									p_str := r[1]
									p_str = strings.Replace(p_str, ",", "", -1)
									SetHome(i.User.Name, p_str)
									mm.Output("tell " + i.User.Name + " Set your home to " + p_str)
									return true
								}
							}
							return false
						})
						mm.Output("tp " + i.User.Name + " ~ ~ ~")
						return true
					}
					return false
				})
				// Add !home listener
				mm.listeners = append(mm.listeners, func(i *Message) bool {
					if i.User.Name != "" && i.Text == "!home\n" {
						home_str, found := GetHome(i.User.Name)
						if found {
							mm.Output("tp " + i.User.Name + " " + home_str)
						} else {
							mm.Output("tell " + i.User.Name + " I don't know where your home is. Set it to your current position by typing '!set home'")
						}
					}
					return false
				})
			} else if opt_name == "visit" {
				c.FeatureTPVisit = opt_enabled
			}
		}

		c.Users = make([]*User, 0, 10)
		u, _ := j.GetObjectArray("users")
		for _, user := range u {
			user_name, err := user.GetString("name")
			if err == nil && user_name != "" {
				user_home, _ := user.GetString("home")
				user_porch, _ := user.GetString("porch")
				us := NewUser(user_name)
				// TODO: Check if this user is an op on the server
				us.Home = user_home
				us.Porch = user_porch
				c.Users = append(c.Users, us)
				c.U = *us
			}
		}
		fmt.Printf("Loaded %d Users\n", len(c.Users))
	}
}

func WriteConfig() {
	// Generate the JSON string for the config file
	d := "{\"options\":["
	// Output options array
	d = d + "{\"name\":\"home\",\"enabled\":"
	if c.FeatureTPHome {
		d = d + "true"
	} else {
		d = d + "false"
	}
	d = d + "},{\"name\":\"visit\",\"enabled\":"
	if c.FeatureTPVisit {
		d = d + "true"
	} else {
		d = d + "false"
	}
	d = d + "}],\"users\":["
	d = d + c.U.ToJSONString()
	// Output users array
	/*
		num_users := len(c.Users)
		fmt.Printf("Number of Users: %d", num_users)
		for i := 0; i < num_users; i++ {
			user := c.Users[i]
			//	for _, user := range c.Users {
			fmt.Println(d)
			num_users--
			d = d + user.ToJSONString()
			if user.Name != "" {
				if num_users > 0 {
					d = d + ","
				}
			}
		}
	*/
	d = d + "]}"
	do := []byte(d)
	ioutil.WriteFile("mc_man.config", do, 0644)
}

func SetHome(user string, loc string) {
	/*
		u, idx := FindUser(user)
		if idx == -1 {
			u = NewUser(user)
			c.Users = append(c.Users, u)
			idx = len(c.Users) - 1
		}
		u.Home = strings.Replace(loc, "\n", "", -1)
		// Replace the user in the Users array
		c.Users[idx] = u
	*/
	c.U.Home = strings.Replace(loc, "\n", "", -1)
	WriteConfig()
}

func GetHome(user string) (string, bool) {
	/*
		u, idx := FindUser(user)
		if idx == -1 || u.Home == "" {
			return "", false
		}
	*/
	return c.U.Home, true
}

func SetPorch(user string, loc string) {
	/*
		u, idx := FindUser(user)
		if idx == -1 {
			u = NewUser(user)
			c.Users = append(c.Users, u)
			idx = len(c.Users) - 1
		}
		u.Porch = strings.Replace(loc, "\n", "", -1)
		c.Users[idx] = u
	*/
	c.U.Porch = strings.Replace(loc, "\n", "", -1)
	WriteConfig()
}

func GetPorch(user string) (string, bool) {
	/*
		u, idx := FindUser(user)
		if idx > -1 || u.Porch == "" {
			return "", false
		}
		return u.Porch, true
	*/
	return c.U.Porch, true
}

func FindUser(name string) (*User, int) {
	for i, user := range c.Users {
		if user.Name == name {
			return user, i
		}
	}
	return nil, -1
}
